@@ -2,7 +2,7 @@
 import pandas as pd
 from dateutil.relativedelta import relativedelta
 
-import utils
+from amendment_forecast import utils
 from amendment_forecast.models import DATE_COLUMN_NAME, VALUE_COLUMN_NAME, initialize_model
 
 FREQUENCY_MAPPING = {
@@ -31,7 +31,8 @@ def run_forecast_ensemble(
         aggregate_operation="sum",
         training_holdout_pct=0.3,
         frequency="week",
-        period_format=None):
+        period_format=None,
+        model_list=None):
     # Initialize with copy of input date
     df = dataframe.copy()
     df[DATE_COLUMN_NAME] = pd.to_datetime(df[date_column])
@@ -54,14 +55,17 @@ def run_forecast_ensemble(
     train_end_date = get_train_end_date(df, training_holdout_pct)
 
     # Assemble ensemble of models
-    named_model_list = [
-        # "GreyKite",
-        # "FBProphet",
-        # "Naive",
-        # "XGBoost",
-        # "RandomForest",
-        "SARIMA"
-    ]
+    if model_list:
+        named_model_list = model_list
+    else:
+        named_model_list = [
+            "GreyKite",
+            "FBProphet",
+            "Naive",
+            "XGBoost",
+            "RandomForest",
+            "SARIMA"
+        ]
 
     # For each model, run a full evaluation and add to the ensemble results
     ensemble = []
@@ -80,7 +84,10 @@ def run_forecast_ensemble(
         model_dict["forecast_timestamps"] = results.pop("forecast_timestamps")
         model_dict["forecast_values"] = results.pop("forecast_values")
         model_dict["performance_metrics"] = results
-        model_dict["weight"] = model_dict["performance_metrics"]["r2"]
+        weight = model_dict["performance_metrics"]["r2"]
+        if weight < 0:
+            weight = 0
+        model_dict["weight"] = weight
         ensemble.append(model_dict)
 
     # Combine outputs to calculate ensemble effectiveness
@@ -89,19 +96,25 @@ def run_forecast_ensemble(
     ensemble_forecast_list = []
     for model in ensemble:
         model["weight"] = model["weight"] / total_weight
-        predictions = pd.Series(model["prediction_values"]).reset_index(drop=True) * model["weight"]
-        forecast = pd.Series(model["forecast_values"]).reset_index(drop=True) * model["weight"]
-        actuals = pd.Series(model["actual_values"])
-        ensemble_predictions_list.append(predictions)
-        ensemble_forecast_list.append(forecast)
+        if model["weight"] > 0:
+            predictions = pd.Series(model["prediction_values"]).reset_index(drop=True) * model["weight"]
+            forecast = pd.Series(model["forecast_values"]).reset_index(drop=True) * model["weight"]
+            actuals = pd.Series(model["actual_values"])
+            ensemble_predictions_list.append(predictions)
+            ensemble_forecast_list.append(forecast)
     ensemble_train_timestamps = model["train_timestamps"]
     ensemble_predictions = sum(ensemble_predictions_list)
     ensemble_actuals = model["actual_values"]
     ensemble_forecast_timestamps = model["forecast_timestamps"]
     ensemble_forecast = sum(ensemble_forecast_list)
 
-    performance_metrics = utils.get_model_statistics(ensemble_predictions, ensemble_actuals)
-    consolidated_metrics = utils.consolidate_scores(performance_metrics, ensemble_actuals.mean())
+    ensemble_train_df = pd.DataFrame({
+        "timestamps": ensemble_train_timestamps,
+        "actuals": ensemble_actuals,
+        "predictions": ensemble_predictions})
+    ensemble_train_df = ensemble_train_df[~ensemble_train_df.predictions.isnull()]
+    performance_metrics = utils.get_model_statistics(ensemble_train_df["predictions"], ensemble_train_df["actuals"])
+    consolidated_metrics = utils.consolidate_scores(performance_metrics, ensemble_train_df["actuals"].mean())
 
     degraded_accuracies = {}
     training_years = (df[DATE_COLUMN_NAME].max() - df[DATE_COLUMN_NAME].min()).days / 365
@@ -114,7 +127,7 @@ def run_forecast_ensemble(
                     multiplier *= 0.5
                 elif yy > training_years:
                     multiplier *= 0.95
-        degraded_accuracies[year] = multiplier * consolidated_metrics["composite"]
+        degraded_accuracies[year] = multiplier * consolidated_metrics["accuracy"]
 
     ensemble.append({
         "name": "ensemble",
