@@ -4,7 +4,7 @@ import pandas as pd
 from datetime import datetime
 from dateutil.relativedelta import relativedelta
 
-from greykite.framework.templates.autogen.forecast_config import ForecastConfig, MetadataParam
+from greykite.framework.templates.autogen.forecast_config import ForecastConfig, MetadataParam, EvaluationPeriodParam
 from greykite.framework.templates.forecaster import Forecaster
 from greykite.framework.templates.model_templates import ModelTemplateEnum
 from xgboost import XGBRegressor
@@ -62,7 +62,7 @@ class GreyKite(BaseModel):
             time_col=DATE_COLUMN_NAME,
             value_col=VALUE_COLUMN_NAME)
 
-    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str) -> tuple:
+    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         """Runs rolling prediction using the model to return actual and predicted values
 
         Args:
@@ -73,20 +73,30 @@ class GreyKite(BaseModel):
         """
         forecaster = Forecaster()
         self.metadata.freq = frequency
-        train_mask = dataframe[DATE_COLUMN_NAME] <= train_end
+        forecast_horizon=len(forecast_period_list)
+        if forecast_horizon >= (len(dataframe) / 3):
+            test_horizon = (len(dataframe) / 3) - 1
+        else:
+            test_horizon = forecast_horizon
         result = forecaster.run_forecast_config(  # result is also stored as `forecaster.forecast_result`.
             df=dataframe,
             config=ForecastConfig(
+                evaluation_period_param=EvaluationPeriodParam(test_horizon=test_horizon),
                 model_template=ModelTemplateEnum.SILVERKITE.name,
                 coverage=0.95,
                 metadata_param=self.metadata,
-                forecast_horizon=len(dataframe) - sum(train_mask)
+                forecast_horizon=len(forecast_period_list)
             )
         )
         backtest = result.backtest.df_test[~result.backtest.df_test["actual"].isnull()]
         backtest = backtest[backtest[DATE_COLUMN_NAME] > train_end]
+        backtest = backtest.set_index("period")
+        forecast_df = result.forecast.df
+        forecast_df = forecast_df.set_index("period")
+        mask = forecast_df["actual"].isnull()
+        forecast = forecast_df.loc[mask, "forecast"]
 
-        return backtest["actual"], backtest["forecast"]
+        return backtest["actual"], backtest["forecast"], forecast
 
 
 class FBProphet(BaseModel):
@@ -96,7 +106,7 @@ class FBProphet(BaseModel):
         super().__init__()
         self.metadata = MetadataParam(time_col=DATE_COLUMN_NAME, value_col=VALUE_COLUMN_NAME)
 
-    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str) -> tuple:
+    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         """Runs rolling prediction using the model to return actual and predicted values
 
         Args:
@@ -107,20 +117,30 @@ class FBProphet(BaseModel):
         """
         forecaster = Forecaster()
         self.metadata.freq = frequency
-        train_mask = dataframe[DATE_COLUMN_NAME] <= train_end
+        forecast_horizon = len(forecast_period_list)
+        if forecast_horizon >= (len(dataframe) / 3):
+            test_horizon = (len(dataframe) / 3) - 1
+        else:
+            test_horizon = forecast_horizon
         result = forecaster.run_forecast_config(  # result is also stored as `forecaster.forecast_result`.
             df=dataframe,
             config=ForecastConfig(
+                evaluation_period_param=EvaluationPeriodParam(test_horizon=test_horizon),
                 model_template=ModelTemplateEnum.PROPHET.name,
                 coverage=0.95,
                 metadata_param=self.metadata,
-                forecast_horizon=len(dataframe) - sum(train_mask)
+                forecast_horizon=len(forecast_period_list)
             )
         )
         backtest = result.backtest.df_test[~result.backtest.df_test["actual"].isnull()]
         backtest = backtest[backtest[DATE_COLUMN_NAME] > train_end]
+        backtest = backtest.set_index("period")
+        forecast_df = result.forecast.df
+        forecast_df = forecast_df.set_index("period")
+        mask = forecast_df["actual"].isnull()
+        forecast = forecast_df.loc[mask, "forecast"]
 
-        return backtest["actual"], backtest["forecast"]
+        return backtest["actual"], backtest["forecast"], forecast
 
 
 class Naive(BaseModel):
@@ -131,18 +151,13 @@ class Naive(BaseModel):
         self.growth_comparison_period_in_years = 1
         self.minimum_training_years = 2
 
-    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str) -> tuple:
+    def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         # Determine training and testing periods to create test and training dataframes
         comparison_period_relativedelta = relativedelta(years=self.growth_comparison_period_in_years)
         train_mask = dataframe[DATE_COLUMN_NAME] <= train_end
         dataframe_test = dataframe[~train_mask]
 
-        # Loop through individual values
-        actual = []
-        predicted = []
-        for index, row in dataframe_test.head().iterrows():
-            # Determine window to evaluate against for the test datapoint
-            prediction_date = row[DATE_COLUMN_NAME]
+        def run_prediction(prediction_date):
             base_growth_period_start = prediction_date - (2 * comparison_period_relativedelta)
             comparison_growth_period_start = prediction_date - comparison_period_relativedelta
 
@@ -162,13 +177,29 @@ class Naive(BaseModel):
             # Determine value to use as the base for the estimation
             previous_value_mask = dataframe[DATE_COLUMN_NAME] == comparison_growth_period_start
             previous_value = dataframe.loc[previous_value_mask, VALUE_COLUMN_NAME].sum()
-            predicted_value = previous_value * growth_rate
+
+            return previous_value * growth_rate
+
+        # Loop through individual values
+        actual = []
+        predicted = []
+        for index, row in dataframe_test.iterrows():
+            # Determine window to evaluate against for the test datapoint
+            prediction_date = row[DATE_COLUMN_NAME]
+            predicted_value = run_prediction(prediction_date)
 
             # Save prediction
             actual.append(dataframe.loc[index, VALUE_COLUMN_NAME])
             predicted.append(predicted_value)
 
-        return actual, predicted
+        forecast = []
+        for timestamp in forecast_period_list:
+            forecast_value = run_prediction(timestamp)
+            forecast.append(forecast_value)
+            new_row = pd.DataFrame({DATE_COLUMN_NAME: [timestamp], VALUE_COLUMN_NAME: [forecast_value]})
+            dataframe = pd.concat([dataframe, new_row]).reset_index(drop=True)
+
+        return actual, predicted, forecast
 
 
 class XGBoost(BaseModel):
@@ -387,17 +418,20 @@ class SARIMA(BaseModel):
         return yhat.iloc[0]
 
     def full_forecast(self, train_df, forecast_period_list):
-        # fit a SARIMA model and make a one step prediction
-        train_y = train_df[VALUE_COLUMN_NAME]
-
-        # fit model
-        model = SARIMAX(train_y)
-        model = model.fit(disp=False)
-
         # make a one-step prediction
-        yhat = model.forecast(steps=len(forecast_period_list))
+        forecast = pd.Series()
+        # # Step over each time period and get result
+        for period in forecast_period_list:
+            # fit model on history and make a prediction
+            yhat = self.step_forecast(train_df)
 
-        return yhat
+            # store forecast in list of predictions
+            forecast = forecast.append(pd.Series(index=[period], data=[yhat]))
+
+            # add actual observation to history for the next loop
+            train_df = pd.concat([train_df, pd.DataFrame({DATE_COLUMN_NAME: [period], VALUE_COLUMN_NAME: [yhat]})])
+
+        return forecast
 
     def run(self, dataframe: pd.DataFrame, train_end: datetime, frequency: str, forecast_period_list: list) -> tuple:
         dataframe = dataframe.copy()
@@ -411,22 +445,20 @@ class SARIMA(BaseModel):
         train = dataframe[train_mask]
         test = dataframe[~train_mask]
 
-        # # Step over each time period and get result
-        # for index, row in test.iterrows():
-        #     # fit model on history and make a prediction
-        #     yhat = self.step_forecast(train)
-        #
-        #     # store forecast in list of predictions
-        #     predicted.append(yhat)
-        #
-        #     # add actual observation to history for the next loop
-        #     train = pd.concat([train, pd.DataFrame(row).T])
+        # Step over each time period and get result
+        for index, row in test.iterrows():
+            # fit model on history and make a prediction
+            yhat = self.step_forecast(train)
+
+            # store forecast in list of predictions
+            predicted.append(yhat)
+
+            # add actual observation to history for the next loop
+            train = pd.concat([train, pd.DataFrame(row).T])
 
         # Run full forecast
         forecast_predictions = self.full_forecast(dataframe, forecast_period_list)
 
-        predicted = [1.0]*len(test[VALUE_COLUMN_NAME])
-
         return test[VALUE_COLUMN_NAME], predicted, forecast_predictions
 
 
